// 
// File:        ArrayTestDriver.java
// Copyright:   (c) 2001 The Regents of the University of California
// Release:     $Name: release-0-8-6 $
// Revision:    @(#) $Revision: 1.4 $
// Date:        $Date: 2002/08/14 23:17:01 $
// Description: array regression test case for Java calling other languages
// 

/**
 * The following class runs the array regression test cases for Java.
 */
public class ArrayTestDriver {
  private final static int FAIL        = 0;
  private final static int XFAIL       = 1;
  private final static int PASS        = 2;
  private final static int XPASS       = 3;
  private final static int UNSUPPORTED = 4;

  private final static String[] s_results = {
    "FAIL",
    "XFAIL",
    "PASS",
    "XPASS",
    "UNSUPPORTED"
  };

  private final static int ONE_D_SIZE = 345;
  private final static int TEST_DIM1 = 13;
  private final static int TEST_DIM2 = 16;

  private static int s_part = 0;
  private static int s_result = PASS;

  /**
   * Check the results of the test case.
   */
  private static void check(int expected, boolean pass, String test) {
    System.out.println("PART " + String.valueOf(++s_part));
    System.out.println("COMMENT " + test);
    System.out.print("RESULT " + String.valueOf(s_part) + " ");

    if (expected == UNSUPPORTED) {
      System.out.println(s_results[expected]);
      if (s_result == PASS) {
        s_result = XFAIL;
      }
    } else if (expected == PASS) {
      System.out.println(s_results[pass ? PASS : FAIL]);
      if (!pass) {
        s_result = FAIL;
      }
    } else if (expected == XFAIL) {
      System.out.println(s_results[pass ? XPASS : XFAIL]);
      if (pass) {
        s_result = FAIL;
      } else if (s_result > XFAIL) {
        s_result = XFAIL;
      }
    }
  }

  /**
   * Calculate a float power of two by repeated multiplication.  This is
   * the exact operation carried out in the stubs.  Make sure that the answer
   * is bit reproducable.
   */
  private static float fpow2(int p) {
    float result = 1.0F;
    if (p >= 0) {
      while ((p--) > 0) {
        result *= 2.0F;
      }
    } else {
      while ((p++) < 0) {
        result *= 0.5F;
      }
    }
    return result;
  }

  /**
   * Calculate a double power of two by repeated multiplication.  This is
   * the exact operation carried out in the stubs.  Make sure that the answer
   * is bit reproducable.
   */
  private static double dpow2(int p) {
    double result = 1.0;
    if (p >= 0) {
      while ((p--) > 0) {
        result *= 2.0;
      }
    } else {
      while ((p++) < 0) {
        result *= 0.5;
      }
    }
    return result;
  }

  /**
   * Create a float test array.
   */
  private static SIDL.Float.Array2 makeArrayFloat(int rows, int columns) {
    SIDL.Float.Array2 array = new SIDL.Float.Array2(rows, columns);
    for (int i = 0; i < rows; i++) {
      for (int j = 0; j < columns; j++) {
        array.set(i, j, fpow2(i-j));
      }
    }
    return array;
  }

  /**
   * Create a float complex test array.
   */
  private static SIDL.FloatComplex.Array2 makeArrayFloatComplex(
      int rows, int columns) {
    SIDL.FloatComplex.Array2 array =
      new SIDL.FloatComplex.Array2(rows, columns);
    for (int i = 0; i < rows; i++) {
      for (int j = 0; j < columns; j++) {
        array.set(i, j, new SIDL.FloatComplex(fpow2(i), fpow2(-j)));
      }
    }
    return array;
  }

  /**
   * Create a double test array.
   */
  private static SIDL.Double.Array2 makeArrayDouble(int rows, int columns) {
    SIDL.Double.Array2 array = new SIDL.Double.Array2(rows, columns);
    for (int i = 0; i < rows; i++) {
      for (int j = 0; j < columns; j++) {
        array.set(i, j, dpow2(i-j));
      }
    }
    return array;
  }

  /**
   * Create a double complex test array.
   */
  private static SIDL.DoubleComplex.Array2 makeArrayDoubleComplex(
      int rows, int columns) {
    SIDL.DoubleComplex.Array2 array =
      new SIDL.DoubleComplex.Array2(rows, columns);
    for (int i = 0; i < rows; i++) {
      for (int j = 0; j < columns; j++) {
        array.set(i, j, new SIDL.DoubleComplex(dpow2(i), dpow2(-j)));
      }
    }
    return array;
  }

  /**
   * The main test driver takes no command-line arguments and runs the
   * regression tests.
   */
  public static void main(String args[]) {
    try {

      /*
       * Begin the test
       */

      System.out.println("NPARTS -1");

      s_part   = 0;
      s_result = PASS;

      /*
       * Boolean array tests
       */

      if (true) {
        SIDL.Boolean.Array1 array = ArrayTest.ArrayOps.createBool(ONE_D_SIZE);
        check(PASS,
              (array != null),
             "(array != null)");
        check(PASS,
              (ArrayTest.ArrayOps.checkBool(array) == true),
             "(ArrayTest.ArrayOps.checkBool(array) == true)");
        check(PASS,
              (ArrayTest.ArrayOps.reverseBool(array, true) == true),
             "(ArrayTest.ArrayOps.reverseBool(array, true) == true)");

        ArrayTest.ArrayOps.makeBool(218, array);
        check(PASS,
              (ArrayTest.ArrayOps.checkBool(array) == true),
             "(ArrayTest.ArrayOps.checkBool(array) == true)");
        check(PASS,
              (ArrayTest.ArrayOps.reverseBool(array, false) == true),
             "(ArrayTest.ArrayOps.reverseBool(array, false) == true)");
        check(PASS,
              (ArrayTest.ArrayOps.checkBool(array) == false),
             "(ArrayTest.ArrayOps.checkBool(array) == false)");

        ArrayTest.ArrayOps.makeBool(9, array);
        check(PASS,
              (ArrayTest.ArrayOps.reverseBool(array, false) == true),
             "(ArrayTest.ArrayOps.reverseBool(array, false) == true)");
        check(PASS,
              (ArrayTest.ArrayOps.checkBool(array) == true),
             "(ArrayTest.ArrayOps.checkBool(array) == true)");
      }

      /*
       * Character array tests
       */

      if (true) {
        SIDL.Character.Array1 array = ArrayTest.ArrayOps.createChar(ONE_D_SIZE);
        check(PASS,
              (array != null),
             "(array != null)");
        check(PASS,
              (ArrayTest.ArrayOps.checkChar(array) == true),
             "(ArrayTest.ArrayOps.checkChar(array) == true)");
        check(PASS,
              (ArrayTest.ArrayOps.reverseChar(array, true) == true),
             "(ArrayTest.ArrayOps.reverseChar(array, true) == true)");

        ArrayTest.ArrayOps.makeChar(218, array);
        check(PASS,
              (ArrayTest.ArrayOps.checkChar(array) == true),
             "(ArrayTest.ArrayOps.checkChar(array) == true)");
        check(PASS,
              (ArrayTest.ArrayOps.reverseChar(array, false) == true),
             "(ArrayTest.ArrayOps.reverseChar(array, false) == true)");
        check(PASS,
              (ArrayTest.ArrayOps.checkChar(array) == false),
             "(ArrayTest.ArrayOps.checkChar(array) == false)");
      }

      /*
       * Integer array tests
       */

      if (true) {
        SIDL.Integer.Array1 array = ArrayTest.ArrayOps.createInt(ONE_D_SIZE);
        check(PASS,
              (array != null),
             "(array != null)");
        check(PASS,
              (ArrayTest.ArrayOps.checkInt(array) == true),
             "(ArrayTest.ArrayOps.checkInt(array) == true)");
        check(PASS,
              (ArrayTest.ArrayOps.reverseInt(array, true) == true),
             "(ArrayTest.ArrayOps.reverseInt(array, true) == true)");

        ArrayTest.ArrayOps.makeInt(218, array);
        check(PASS,
              (ArrayTest.ArrayOps.checkInt(array) == true),
             "(ArrayTest.ArrayOps.checkInt(array) == true)");
        check(PASS,
              (ArrayTest.ArrayOps.reverseInt(array, false) == true),
             "(ArrayTest.ArrayOps.reverseInt(array, false) == true)");
        check(PASS,
              (ArrayTest.ArrayOps.checkInt(array) == false),
             "(ArrayTest.ArrayOps.checkInt(array) == false)");
      }

      /*
       * Long array tests
       */

      if (true) {
        SIDL.Long.Array1 array = ArrayTest.ArrayOps.createLong(ONE_D_SIZE);
        check(PASS,
              (array != null),
             "(array != null)");
        check(PASS,
              (ArrayTest.ArrayOps.checkLong(array) == true),
             "(ArrayTest.ArrayOps.checkLong(array) == true)");
        check(PASS,
              (ArrayTest.ArrayOps.reverseLong(array, true) == true),
             "(ArrayTest.ArrayOps.reverseLong(array, true) == true)");

        ArrayTest.ArrayOps.makeLong(218, array);
        check(PASS,
              (ArrayTest.ArrayOps.checkLong(array) == true),
             "(ArrayTest.ArrayOps.checkLong(array) == true)");
        check(PASS,
              (ArrayTest.ArrayOps.reverseLong(array, false) == true),
             "(ArrayTest.ArrayOps.reverseLong(array, false) == true)");
        check(PASS,
              (ArrayTest.ArrayOps.checkLong(array) == false),
             "(ArrayTest.ArrayOps.checkLong(array) == false)");
      }

      /*
       * String array tests
       */

      if (true) {
        SIDL.String.Array1 array = ArrayTest.ArrayOps.createString(ONE_D_SIZE);
        check(PASS,
              (array != null),
             "(array != null)");
        check(PASS,
              (ArrayTest.ArrayOps.checkString(array) == true),
             "(ArrayTest.ArrayOps.checkString(array) == true)");
        check(PASS,
              (ArrayTest.ArrayOps.reverseString(array, true) == true),
             "(ArrayTest.ArrayOps.reverseString(array, true) == true)");

        ArrayTest.ArrayOps.makeString(218, array);
        check(PASS,
              (ArrayTest.ArrayOps.checkString(array) == true),
             "(ArrayTest.ArrayOps.checkString(array) == true)");
        check(PASS,
              (ArrayTest.ArrayOps.reverseString(array, false) == true),
             "(ArrayTest.ArrayOps.reverseString(array, false) == true)");
        check(PASS,
              (ArrayTest.ArrayOps.checkString(array) == false),
             "(ArrayTest.ArrayOps.checkString(array) == false)");
      }

      /*
       * Double array tests
       */

      if (true) {
        SIDL.Double.Array1 array = ArrayTest.ArrayOps.createDouble(ONE_D_SIZE);
        check(PASS,
              (array != null),
             "(array != null)");
        check(PASS,
              (ArrayTest.ArrayOps.checkDouble(array) == true),
             "(ArrayTest.ArrayOps.checkDouble(array) == true)");
        check(PASS,
              (ArrayTest.ArrayOps.reverseDouble(array, true) == true),
             "(ArrayTest.ArrayOps.reverseDouble(array, true) == true)");

        ArrayTest.ArrayOps.makeDouble(218, array);
        check(PASS,
              (ArrayTest.ArrayOps.checkDouble(array) == true),
             "(ArrayTest.ArrayOps.checkDouble(array) == true)");
        check(PASS,
              (ArrayTest.ArrayOps.reverseDouble(array, false) == true),
             "(ArrayTest.ArrayOps.reverseDouble(array, false) == true)");
        check(PASS,
              (ArrayTest.ArrayOps.checkDouble(array) == false),
             "(ArrayTest.ArrayOps.checkDouble(array) == false)");
      }

      /*
       * Float array tests
       */

      if (true) {
        SIDL.Float.Array1 array = ArrayTest.ArrayOps.createFloat(ONE_D_SIZE);
        check(PASS,
              (array != null),
             "(array != null)");
        check(PASS,
              (ArrayTest.ArrayOps.checkFloat(array) == true),
             "(ArrayTest.ArrayOps.checkFloat(array) == true)");
        check(PASS,
              (ArrayTest.ArrayOps.reverseFloat(array, true) == true),
             "(ArrayTest.ArrayOps.reverseFloat(array, true) == true)");

        ArrayTest.ArrayOps.makeFloat(218, array);
        check(PASS,
              (ArrayTest.ArrayOps.checkFloat(array) == true),
             "(ArrayTest.ArrayOps.checkFloat(array) == true)");
        check(PASS,
              (ArrayTest.ArrayOps.reverseFloat(array, false) == true),
             "(ArrayTest.ArrayOps.reverseFloat(array, false) == true)");
        check(PASS,
              (ArrayTest.ArrayOps.checkFloat(array) == false),
             "(ArrayTest.ArrayOps.checkFloat(array) == false)");
      }

      /*
       * FloatComplex array tests
       */

      if (true) {
        SIDL.FloatComplex.Array1 array =
          ArrayTest.ArrayOps.createFcomplex(ONE_D_SIZE);
        check(PASS,
              (array != null),
             "(array != null)");
        check(PASS,
              (ArrayTest.ArrayOps.checkFcomplex(array) == true),
             "(ArrayTest.ArrayOps.checkFcomplex(array) == true)");
        check(PASS,
              (ArrayTest.ArrayOps.reverseFcomplex(array, true) == true),
             "(ArrayTest.ArrayOps.reverseFcomplex(array, true) == true)");

        ArrayTest.ArrayOps.makeFcomplex(218, array);
        check(PASS,
              (ArrayTest.ArrayOps.checkFcomplex(array) == true),
             "(ArrayTest.ArrayOps.checkFcomplex(array) == true)");
        check(PASS,
              (ArrayTest.ArrayOps.reverseFcomplex(array, false) == true),
             "(ArrayTest.ArrayOps.reverseFcomplex(array, false) == true)");
        check(PASS,
              (ArrayTest.ArrayOps.checkFcomplex(array) == false),
             "(ArrayTest.ArrayOps.checkFcomplex(array) == false)");
      }

      /*
       * DoubleComplex array tests
       */

      if (true) {
        SIDL.DoubleComplex.Array1 array =
          ArrayTest.ArrayOps.createDcomplex(ONE_D_SIZE);
        check(PASS,
              (array != null),
             "(array != null)");
        check(PASS,
              (ArrayTest.ArrayOps.checkDcomplex(array) == true),
             "(ArrayTest.ArrayOps.checkDcomplex(array) == true)");
        check(PASS,
              (ArrayTest.ArrayOps.reverseDcomplex(array, true) == true),
             "(ArrayTest.ArrayOps.reverseDcomplex(array, true) == true)");

        ArrayTest.ArrayOps.makeDcomplex(218, array);
        check(PASS,
              (ArrayTest.ArrayOps.checkDcomplex(array) == true),
             "(ArrayTest.ArrayOps.checkDcomplex(array) == true)");
        check(PASS,
              (ArrayTest.ArrayOps.reverseDcomplex(array, false) == true),
             "(ArrayTest.ArrayOps.reverseDcomplex(array, false) == true)");
        check(PASS,
              (ArrayTest.ArrayOps.checkDcomplex(array) == false),
             "(ArrayTest.ArrayOps.checkDcomplex(array) == false)");
      }

      /*
       * Object array tests
       */

      if (true) {
        check(XFAIL, false, "Create an array of objects");
        check(XFAIL, false, "Check an array of objects");
      }

      /*
       * Create and check two dimensional array for double
       */

      if (true) {
        SIDL.Double.Array2 array = makeArrayDouble(TEST_DIM1,TEST_DIM2);
        check(PASS,
              (ArrayTest.ArrayOps.check2Double(array) == true),
             "(ArrayTest.ArrayOps.check2Double(array) == true)");
      }

      /*
       * Create and check two dimensional array for float
       */

      if (true) {
        SIDL.Float.Array2 array = makeArrayFloat(TEST_DIM1,TEST_DIM2);
        check(PASS,
              (ArrayTest.ArrayOps.check2Float(array) == true),
             "(ArrayTest.ArrayOps.check2Float(array) == true)");
      }

      /*
       * Create and check two dimensional array for double complex
       */

      if (true) {
        SIDL.DoubleComplex.Array2 array = makeArrayDoubleComplex(TEST_DIM1,TEST_DIM2);
        check(PASS,
              (ArrayTest.ArrayOps.check2Dcomplex(array) == true),
             "(ArrayTest.ArrayOps.check2Dcomplex(array) == true)");
      }

      /*
       * Create and check two dimensional array for float complex
       */

      if (true) {
        SIDL.FloatComplex.Array2 array = makeArrayFloatComplex(TEST_DIM1,TEST_DIM2);
        check(PASS,
              (ArrayTest.ArrayOps.check2Fcomplex(array) == true),
             "(ArrayTest.ArrayOps.check2Fcomplex(array) == true)");
      }

      /*
       * Output final test results
       */

      System.out.println("TEST_RESULT " + s_results[s_result]);

      Runtime.getRuntime().exit(0); /* workaround for Linux JVM 1.3.1 bug */
    /*
     * Catch any unexpected exceptions and return a test failure
     */
    } catch (Throwable ex) {
      System.out.println("TEST_RESULT FAIL");
      System.out.println(ex.toString());
      System.exit(1);
    }
  }
}
